/* 
 * unittester.h -- include needed by all test suites
 *
 * This file is part of the CUnitTester pacakge
 * see http://www.gethos.net/opensource/cunit
 *
 * Copyright 2003 by Stewart Gebbie
 * For license terms, see the file COPYING in the top directory.
 */
/* $Id: unittester.h,v 1.14 2005/01/21 17:09:44 stewart Exp $ */
#ifndef UNITESTER_H
#define UNITESTER_H

#include <stdlib.h>

#ifndef __cplusplus
#define true 1
#define false 0
typedef unsigned short bool;
#endif /* __cplusplus */

#define uassert(exp) if (!(exp)) 							\
		 {													\
			 fprintf(stderr,"Assertion (%s) failed %s:%d\n",\
				#exp,__FILE__, __LINE__);					\
			 abort();										\
		 }													\

#define MAXUTESTNAMELEN 100
#define MAXUTESTFILELEN 100
#define MAXUTESTPATHLEN 200
#define MAXUTESTMESGLEN 200

#define UTEST_PASS return true

#define UTEST_ASSERT(exp,msg,...) 									\
		if(!(exp))					 								\
		{ 															\
		    info->logerror(info,__LINE__,__FILE__,msg,##__VA_ARGS__);	\
		    return false; 											\
		}															\

#define UTEST_FAIL(msg,...)	 											\
		if(1)						 								\
		{ 															\
		    info->logerror(info,__LINE__,__FILE__,msg,##__VA_ARGS__);	\
		    return false; 											\
		}															\

#ifndef __cplusplus
#define UTEST_REGISTER(suite)					\
utest_suite* register_suite(void)				\
{												\
	return &suite;								\
}
#else
#define UTEST_REGISTER(suite)					\
extern "C" utest_suite* register_suite(void)	\
{												\
	return (utest_suite*)(&suite);				\
}
#endif


#ifdef __cplusplus
extern "C" {
#endif

typedef struct utest_info_tag
{
    int line;
    char file[MAXUTESTFILELEN];
    char msg[MAXUTESTMESGLEN];
    char suiteName[MAXUTESTNAMELEN];
    char testName[MAXUTESTNAMELEN];
    int status;
    void (*logerror)(struct utest_info_tag*,int,const char*,const char*,...);
    struct utest_info_tag* next;
} utest_info;

typedef struct utest_test_tag
{
    char name[MAXUTESTNAMELEN];
    int (*theTest)(utest_info*);
} utest_test;

#define SUITE_PROLOG					\
    char name[MAXUTESTNAMELEN];			\
    int (*initialiser)(utest_info*);	\
    int (*finaliser)(utest_info*);		\
    int (*setup)(utest_info*);			\
    int (*teardown)(utest_info*);		\

#ifndef MAXTESTSNUMBER
#define MAXTESTSNUMBER 128
#endif

typedef struct utest_suite_tag
{
	SUITE_PROLOG
    utest_test tests[MAXTESTSNUMBER];
} utest_suite;

#ifdef __cplusplus
}
#endif

#ifdef __cplusplus

#define DECLARE_CXX_SUITE(n)					\
typedef struct cxx_utest_suite_tag				\
{												\
	SUITE_PROLOG								\
	utest_test tests[n];						\
} cxx_utest_suite;								\

#endif


#endif /* UNITESTER_H */
